@@ -1,7 +1,7 @@
 // drumkv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,6 +30,8 @@
 
 #include "drumkv1_fx.h"
 
+#include "drumkv1_reverb.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -50,7 +52,7 @@ const uint8_t  MAX_NOTES  = 128;
 const uint8_t  MAX_GROUP  = 128;
 
 const float MIN_ENV_MSECS = 2.0f;		// min 2msec per stage
-const float MAX_ENV_MSECS = 2000.0f;	// max 2 sec per stage
+const float MAX_ENV_MSECS = 2000.0f;	// max 2 sec per stage (default)
 
 const float DETUNE_SCALE  = 0.5f;
 const float PHASE_SCALE   = 0.5f;
@@ -211,7 +213,7 @@ struct drumkv1_env
 				p->frames = min_frames;
 			p->phase = 0.0f;
 			p->delta = 1.0f / float(p->frames);
-			p->c1 = p->value - 1.0f;
+			p->c1 = -(p->value);
 			p->c0 = p->value;
 		}
 		else if (p->stage == Decay2) {
@@ -310,6 +312,7 @@ struct drumkv1_gen
 	float *group;
 	float *coarse;
 	float *fine;
+	float *envtime, envtime0;
 };
 
 
@@ -427,6 +430,18 @@ struct drumkv1_del
 };
 
 
+// reverb
+
+struct drumkv1_rev
+{
+	float *wet;
+	float *room;
+	float *damp;
+	float *feedb;
+	float *width;
+};
+
+
 // dynamic(compressor/limiter)
 
 struct drumkv1_dyn
@@ -623,11 +638,12 @@ public:
 
 	drumkv1_element element;
 
-	drumkv1_sample gen1_sample;
+	drumkv1_sample  gen1_sample;
+	drumkv1_wave_lf lfo1_wave;
+
 	drumkv1_gen    gen1;
 	drumkv1_dcf    dcf1;
 	drumkv1_lfo    lfo1;
-	drumkv1_wave   lfo1_wave;
 	drumkv1_dca    dca1;
 	drumkv1_out    out1;
 
@@ -637,6 +653,8 @@ public:
 	drumkv1_ramp4  vol1;
 
 	float params[3][drumkv1::NUM_ELEMENT_PARAMS];
+
+	void updateEnvTimes(uint32_t iSampleRate);
 };
 
 
@@ -652,19 +670,35 @@ drumkv1_elem::drumkv1_elem ( uint32_t iSampleRate, int key )
 	// element key (sample note)
 	gen1.sample0 = float(key);
 
-	params[0][drumkv1::GEN1_SAMPLE] = gen1.sample0;
-	params[1][drumkv1::GEN1_SAMPLE] = gen1.sample0;
-	params[2][drumkv1::GEN1_SAMPLE] = gen1.sample0;
+	// max env. stage length (default)
+	gen1.envtime0 = 0.0001f * MAX_ENV_MSECS;
+
+	for (int j = 0; j < 3; ++j) {
+		params[j][drumkv1::GEN1_SAMPLE]  = gen1.sample0;
+		params[j][drumkv1::GEN1_ENVTIME] = gen1.envtime0;
+	}
 
 	// element sample rate
 	gen1_sample.setSampleRate(iSampleRate);
 	lfo1_wave.setSampleRate(iSampleRate);
 
+	updateEnvTimes(iSampleRate);
+}
+
+
+void drumkv1_elem::updateEnvTimes ( uint32_t iSampleRate )
+{
 	// element envelope range times in frames
 	const float srate_ms = 0.001f * float(iSampleRate);
 
+	float envtime_msecs = 10000.0f * gen1.envtime0;
+	if (envtime_msecs < MIN_ENV_MSECS)
+		envtime_msecs = (gen1_sample.length() >> 1) / srate_ms;
+	if (envtime_msecs < MIN_ENV_MSECS)
+		envtime_msecs = MIN_ENV_MSECS + 1.0f;
+
 	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
-	const uint32_t max_frames = uint32_t(srate_ms * MAX_ENV_MSECS);
+	const uint32_t max_frames = uint32_t(srate_ms * envtime_msecs);
 
 	dcf1.env.min_frames = min_frames;
 	dcf1.env.max_frames = max_frames;
@@ -799,6 +833,7 @@ private:
 	drumkv1_fla m_fla;
 	drumkv1_pha m_pha;
 	drumkv1_del m_del;
+	drumkv1_rev m_rev;
 	drumkv1_dyn m_dyn;
 
 	drumkv1_voice **m_voices;
@@ -820,6 +855,8 @@ private:
 	drumkv1_fx_phaser  *m_phaser;
 	drumkv1_fx_delay   *m_delay;
 	drumkv1_fx_comp    *m_comp;
+
+	drumkv1_reverb m_reverb;
 };
 
 
@@ -1119,6 +1156,11 @@ void drumkv1_impl::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 	case drumkv1::DEL1_BPM:       m_del.bpm       = pfParam; break;
 	case drumkv1::DEL1_BPMSYNC:   m_del.bpmsync   = pfParam; break;
 	case drumkv1::DEL1_BPMHOST:   m_del.bpmhost   = pfParam; break;
+	case drumkv1::REV1_WET:       m_rev.wet       = pfParam; break;
+	case drumkv1::REV1_ROOM:      m_rev.room      = pfParam; break;
+	case drumkv1::REV1_DAMP:      m_rev.damp      = pfParam; break;
+	case drumkv1::REV1_FEEDB:     m_rev.feedb     = pfParam; break;
+	case drumkv1::REV1_WIDTH:     m_rev.width     = pfParam; break;
 	case drumkv1::DYN1_COMPRESS:  m_dyn.compress  = pfParam; break;
 	case drumkv1::DYN1_LIMITER:   m_dyn.limiter   = pfParam; break;
 	default:
@@ -1162,6 +1204,11 @@ float *drumkv1_impl::paramPort ( drumkv1::ParamIndex index )
 	case drumkv1::DEL1_BPM:       pfParam = m_del.bpm;       break;
 	case drumkv1::DEL1_BPMSYNC:   pfParam = m_del.bpmsync;   break;
 	case drumkv1::DEL1_BPMHOST:   pfParam = m_del.bpmhost;   break;
+	case drumkv1::REV1_WET:       pfParam = m_rev.wet;       break;
+	case drumkv1::REV1_ROOM:      pfParam = m_rev.room;      break;
+	case drumkv1::REV1_DAMP:      pfParam = m_rev.damp;      break;
+	case drumkv1::REV1_FEEDB:     pfParam = m_rev.feedb;     break;
+	case drumkv1::REV1_WIDTH:     pfParam = m_rev.width;     break;
 	case drumkv1::DYN1_COMPRESS:  pfParam = m_dyn.compress;  break;
 	case drumkv1::DYN1_LIMITER:   pfParam = m_dyn.limiter;   break;
 	default:
@@ -1352,6 +1399,9 @@ void drumkv1_impl::allSoundOff (void)
 		m_delay[k].reset();
 		m_comp[k].reset();
 	}
+
+	m_reverb.setSampleRate(m_iSampleRate);
+	m_reverb.reset();
 }
 
 
@@ -1434,6 +1484,9 @@ void drumkv1_impl::reset (void)
 	if (m_comp == 0)
 		m_comp = new drumkv1_fx_comp [m_iChannels];
 
+	// reverbs
+	m_reverb.reset();
+
 	allSoundOff();
 //	allControllersOff();
 	allNotesOff();
@@ -1461,11 +1514,12 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 			elem->gen1_sample.reset(note_freq(elem->gen1.sample0));
 		}
 	#endif
-		if (int(*elem->lfo1.shape) != int(elem->lfo1_wave.shape())
-			||  *elem->lfo1.width  !=     elem->lfo1_wave.width()) {
-			elem->lfo1_wave.reset(
-				drumkv1_wave::Shape(*elem->lfo1.shape), *elem->lfo1.width);
+		if (elem->gen1.envtime0 != *elem->gen1.envtime) {
+			elem->gen1.envtime0  = *elem->gen1.envtime;
+			elem->updateEnvTimes(m_iSampleRate);
 		}
+		elem->lfo1_wave.reset_test(
+			drumkv1_wave::Shape(*elem->lfo1.shape), *elem->lfo1.width);
 		elem = elem->next();
 	}
 
@@ -1481,8 +1535,12 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		drumkv1_elem *elem = pv->elem;
 
 		const float lfo1_rate = *elem->lfo1.rate * *elem->lfo1.rate;
-		const float lfo1_freq = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
-		const float modwheel1 = m_ctl.modwheel + PITCH_SCALE * *elem->lfo1.pitch;
+
+		const float lfo1_freq
+			= LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+
+		const float modwheel1
+			= m_ctl.modwheel + PITCH_SCALE * *elem->lfo1.pitch;
 
 		// channel indexes
 
@@ -1611,16 +1669,15 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		m_del.bpm = del_bpmhost;
 	}
 
-	// effects
+	// chorus
+	if (m_iChannels > 1) {
+		m_chorus.process(outs[0], outs[1], nframes, *m_cho.wet,
+			*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
+	}
 
+	// effects
 	for (k = 0; k < m_iChannels; ++k) {
 		float *in = outs[k];
-		float *out = in;
-		// chorus
-		if (k > 0) {
-			m_chorus.process(outs[k - 1], outs[k], nframes, *m_cho.wet,
-				*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
-		}
 		// flanger
 		m_flanger[k].process(in, nframes, *m_fla.wet,
 			*m_fla.delay, *m_fla.feedb, *m_fla.daft * float(k));
@@ -1630,11 +1687,23 @@ void drumkv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		// delay
 		m_delay[k].process(in, nframes, *m_del.wet,
 			*m_del.delay, *m_del.feedb, *m_del.bpm);
+	}
+
+	// reverb
+	if (m_iChannels > 1) {
+		m_reverb.process(outs[0], outs[1], nframes, *m_rev.wet,
+			*m_rev.feedb, *m_rev.room, *m_rev.damp, *m_rev.width);
+	}
+
+	// dynamics
+	for (k = 0; k < m_iChannels; ++k) {
+		float *in = outs[k];
 		// compressor
 		if (int(*m_dyn.compress) > 0)
 			m_comp[k].process(in, nframes);
 		// limiter
 		if (int(*m_dyn.limiter) > 0) {
+			float *out = in;
 			for (uint32_t n = 0; n < nframes; ++n)
 				*out++ = drumkv1_sigmoid(*in++);
 		}
@@ -1845,6 +1914,7 @@ void drumkv1_element::setParamPort ( drumkv1::ParamIndex index, float *pfParam )
 	case drumkv1::GEN1_GROUP:    m_pElem->gen1.group       = pfParam; break;
 	case drumkv1::GEN1_COARSE:   m_pElem->gen1.coarse      = pfParam; break;
 	case drumkv1::GEN1_FINE:     m_pElem->gen1.fine        = pfParam; break;
+	case drumkv1::GEN1_ENVTIME:  m_pElem->gen1.envtime     = pfParam; break;
 	case drumkv1::DCF1_CUTOFF:   m_pElem->dcf1.cutoff      = pfParam; break;
 	case drumkv1::DCF1_RESO:     m_pElem->dcf1.reso        = pfParam; break;
 	case drumkv1::DCF1_TYPE:     m_pElem->dcf1.type        = pfParam; break;
@@ -1892,6 +1962,7 @@ float *drumkv1_element::paramPort ( drumkv1::ParamIndex index )
 	case drumkv1::GEN1_GROUP:    pfParam = m_pElem->gen1.group;      break;
 	case drumkv1::GEN1_COARSE:   pfParam = m_pElem->gen1.coarse;     break;
 	case drumkv1::GEN1_FINE:     pfParam = m_pElem->gen1.fine;       break;
+	case drumkv1::GEN1_ENVTIME:  pfParam = m_pElem->gen1.envtime;    break;
 	case drumkv1::DCF1_CUTOFF:   pfParam = m_pElem->dcf1.cutoff;     break;
 	case drumkv1::DCF1_RESO:     pfParam = m_pElem->dcf1.reso;       break;
 	case drumkv1::DCF1_TYPE:     pfParam = m_pElem->dcf1.type;       break;
