@@ -1,7 +1,7 @@
 // drumkv1.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -71,6 +71,7 @@ public:
 		GEN1_GROUP,
 		GEN1_COARSE,
 		GEN1_FINE,
+		GEN1_ENVTIME,
 		DCF1_CUTOFF,
 		DCF1_RESO,
 		DCF1_TYPE,
@@ -131,6 +132,11 @@ public:
 		DEL1_BPM,
 		DEL1_BPMSYNC,
 		DEL1_BPMHOST,
+		REV1_WET,
+		REV1_ROOM,
+		REV1_DAMP,
+		REV1_FEEDB,
+		REV1_WIDTH,
 		DYN1_COMPRESS,
 		DYN1_LIMITER,
 
