@@ -1,7 +1,7 @@
 // drumkv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -34,27 +34,31 @@
 //-------------------------------------------------------------------------
 // alsa input thread.
 
-#include <pthread.h>
+#include <QThread>
 
 
-class drumkv1_alsa_thread
+class drumkv1_alsa_thread : public QThread
 {
 public:
 
 	drumkv1_alsa_thread(drumkv1_jack *drumk)
-		: m_drumk(drumk), m_running(false) {}
+		: QThread(), m_drumk(drumk), m_running(false) {}
 
 	~drumkv1_alsa_thread()
-		{ m_running = false; wait(); }
+	{	// fake sync and wait
+		if (m_running && isRunning()) do {
+			m_running = false;
+		}
+		while (!wait(100));
+	}
 
-	void start()
-		{ pthread_create(&m_pthread, NULL, drumkv1_alsa_thread::run, this); }
+protected:
 
-	void *run()
+	void run()
 	{
 		snd_seq_t *seq = m_drumk->alsa_seq();
 		if (seq == NULL)
-			return NULL;
+			return;
 
 		m_running = true;
 
@@ -79,29 +83,13 @@ public:
 		}
 
 		m_running = false;
-		return NULL;
 	}
 
-	void wait()
-		{ pthread_join(m_pthread, NULL); }
-
-	void setRunning(bool running)
-		{ m_running = running; }
-	bool isRunning() const
-		{ return m_running; }
-
-protected:
-
-	static void *run ( void *arg )
-		{ return static_cast<drumkv1_alsa_thread *> (arg)->run(); }
-
 private:
 
 	drumkv1_jack *m_drumk;
 
 	bool m_running;
-
-	pthread_t m_pthread;
 };
 
 #endif	// CONFIG_ALSA_MIDI
@@ -126,6 +114,8 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 {
 	m_client = NULL;
 
+	m_activated = false;
+
 	m_audio_ins = NULL;
 	m_audio_outs = NULL;
 
@@ -154,8 +144,8 @@ drumkv1_jack::drumkv1_jack (void) : drumkv1(2)
 
 drumkv1_jack::~drumkv1_jack (void)
 {
-//	deactivate();
-//	close();
+	deactivate();
+	close();
 }
 
 
@@ -167,6 +157,9 @@ jack_client_t *drumkv1_jack::client (void) const
 
 int drumkv1_jack::process ( jack_nframes_t nframes )
 {
+	if (!m_activated)
+		return 0;
+
 	const uint16_t nchannels = channels();
 	float *ins[nchannels], *outs[nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k) {
@@ -194,25 +187,27 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 
 #ifdef CONFIG_JACK_MIDI
 	void *midi_in = ::jack_port_get_buffer(m_midi_in, nframes);
-	uint32_t nevents = ::jack_midi_get_event_count(midi_in);
-	for (uint32_t n = 0; n < nevents; ++n) {
-		jack_midi_event_t event;
-		::jack_midi_event_get(&event, midi_in, n);
-		uint32_t nread = event.time - ndelta;
-		if (nread > 0) {
-			drumkv1::process(ins, outs, nread);
-			for (uint16_t k = 0; k < nchannels; ++k) {
-				ins[k]  += nread;
-				outs[k] += nread;
+	if (midi_in) {
+		const uint32_t nevents = ::jack_midi_get_event_count(midi_in);
+		for (uint32_t n = 0; n < nevents; ++n) {
+			jack_midi_event_t event;
+			::jack_midi_event_get(&event, midi_in, n);
+			uint32_t nread = event.time - ndelta;
+			if (nread > 0) {
+				drumkv1::process(ins, outs, nread);
+				for (uint16_t k = 0; k < nchannels; ++k) {
+					ins[k]  += nread;
+					outs[k] += nread;
+				}
 			}
+			ndelta = event.time;
+			drumkv1::process_midi(event.buffer, event.size);
 		}
-		ndelta = event.time;
-		drumkv1::process_midi(event.buffer, event.size);
 	}
 #endif
 #ifdef CONFIG_ALSA_MIDI
-	jack_nframes_t buffer_size = ::jack_get_buffer_size(m_client);
-	jack_nframes_t frame_time  = ::jack_last_frame_time(m_client);
+	const jack_nframes_t buffer_size = ::jack_get_buffer_size(m_client);
+	const jack_nframes_t frame_time  = ::jack_last_frame_time(m_client);
 	uint8_t event_buffer[1024];
 	jack_midi_event_t event;
 	while (::jack_ringbuffer_peek(m_alsa_buffer,
@@ -225,7 +220,7 @@ int drumkv1_jack::process ( jack_nframes_t nframes )
 		else
 			event_time = buffer_size - event_time;
 		if (event_time > ndelta) {
-			uint32_t nread = event_time - ndelta;
+			const uint32_t nread = event_time - ndelta;
 			if (nread > 0) {
 				drumkv1::process(ins, outs, nread);
 				for (uint16_t k = 0; k < nchannels; ++k) {
@@ -306,7 +301,7 @@ void drumkv1_jack::open ( const char *client_id )
 		m_alsa_buffer = ::jack_ringbuffer_create(
 			1024 * (sizeof(jack_midi_event_t) + 4));
 		m_alsa_thread = new drumkv1_alsa_thread(this);
-		m_alsa_thread->start();
+		m_alsa_thread->start(QThread::TimeCriticalPriority);
 	}
 #endif	// CONFIG_ALSA_MIDI
 
@@ -318,22 +313,28 @@ void drumkv1_jack::open ( const char *client_id )
 
 void drumkv1_jack::activate (void)
 {
-	drumkv1::reset();
-
-	if (m_client) ::jack_activate(m_client);
+	if (!m_activated) {
+		drumkv1::reset();
+		if (m_client) {
+			::jack_activate(m_client);
+			m_activated = true;
+		}
+	}
 }
 
 void drumkv1_jack::deactivate (void)
 {
-	if (m_client) ::jack_deactivate(m_client);
+	if (m_activated) {
+		if (m_client) {
+			m_activated = false;
+			::jack_deactivate(m_client);
+		}
+	}
 }
 
 
 void drumkv1_jack::close (void)
 {
-	if (m_client == NULL)
-		return;
-
 #ifdef CONFIG_ALSA_MIDI
 	// close alsa sequencer client...
 	if (m_alsa_seq) {
@@ -357,7 +358,10 @@ void drumkv1_jack::close (void)
 	//	m_alsa_client = -1;
 		m_alsa_seq = NULL;
 	}
-#endif	// CONFIG_ALSA_MIDI
+#endif
+
+	if (m_client == NULL)
+		return;
 
 #ifdef CONFIG_JACK_MIDI
 	// unregister midi port
@@ -368,7 +372,7 @@ void drumkv1_jack::close (void)
 #endif
 
 	// unregister audio ports
-	uint16_t nchannels = channels();
+	const uint16_t nchannels = channels();
 
 	for (uint16_t k = 0; k < nchannels; ++k) {
 		if (m_audio_outs && m_audio_outs[k]) {
