@@ -1,7 +1,7 @@
 // drumkv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -40,6 +40,7 @@ struct {
 	{ "GEN1_GROUP",     0.0f },
 	{ "GEN1_COARSE",    0.0f },
 	{ "GEN1_FINE",      0.0f },
+	{ "GEN1_ENVTIME",   0.2f },
 	{ "DCF1_CUTOFF",    1.0f }, // 0.5f
 	{ "DCF1_RESO",      0.0f },
 	{ "DCF1_TYPE",      0.0f },
@@ -98,12 +99,17 @@ struct {
 	{ "DEL1_BPM",     180.0f },
 	{ "DEL1_BPMSYNC",   0.0f },
 	{ "DEL1_BPMHOST", 180.0f },
+	{ "REV1_WET",       0.0f },
+	{ "REV1_ROOM",      0.5f },
+	{ "REV1_DAMP",      0.5f },
+	{ "REV1_FEEDB",     0.5f },
+	{ "REV1_WIDTH",     0.0f },
 	{ "DYN1_COMPRESS",  0.0f },
-	{ "DYN1_LIMIT",     1.0f }
+	{ "DYN1_LIMITER",   1.0f }
 };
 
 
-QString drumkv1_param::paramName ( drumkv1::ParamIndex index )
+const char *drumkv1_param::paramName ( drumkv1::ParamIndex index )
 {
 	return drumkv1_default_params[index].name;
 }
