@@ -1,7 +1,7 @@
 // drumkv1_param.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -58,7 +58,7 @@ namespace drumkv1_param
 		const map_path& mapPath = map_path());
 
 	// Default parameter name/value helpers.
-	QString paramName(drumkv1::ParamIndex index);
+	const char *paramName(drumkv1::ParamIndex index);
 	float paramDefaultValue(drumkv1::ParamIndex index);
 };
 
