@@ -1,7 +1,7 @@
 // drumkv1_sample.h
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -75,13 +75,17 @@ public:
 		m_rate0     = float(info.samplerate);
 		m_nframes   = info.frames;
 
+		const uint32_t nsize = m_nframes + 4;
+
 		m_pframes = new float * [m_nchannels];
-		for (uint16_t k = 0; k < m_nchannels; ++k)
-			m_pframes[k] = new float [m_nframes];
+		for (uint16_t k = 0; k < m_nchannels; ++k) {
+			m_pframes[k] = new float [nsize];
+			::memset(m_pframes[k], 0, nsize * sizeof(float));
+		}
 
 		float *buffer = new float [m_nchannels * m_nframes];
 
-		int nread = ::sf_readf_float(file, buffer, m_nframes);
+		const int nread = ::sf_readf_float(file, buffer, m_nframes);
 		if (nread > 0) {
 			const uint32_t n = uint32_t(nread);
 			uint32_t i = 0;
@@ -149,42 +153,7 @@ public:
 
 	// predicate.
 	bool isOver(uint32_t frame) const
-		{ return !m_pframes || (frame >= m_nframes - 4); }
-
-protected:
-
-	// zero-crossing aliasing (channel).
-	uint32_t zero_crossing_k ( uint32_t i, uint16_t k ) const
-	{
-		const float *frames = m_pframes[k];
-		const float v0 = frames[i];
-
-		int dmax = m_nframes - i;
-		if (dmax < int(i))
-			dmax = i;
-
-		int d = 1;
-		while (d < dmax) {
-			const float v1 = frames[i + d];
-			if ((v0 >= 0.0f && v1 < 0.0f) ||
-				(v1 >= 0.0f && v0 < 0.0f))
-				return (i + d);
-			d = -d;
-			if (d > 0)
-				++d;
-		}
-
-		return (i + d < m_nframes ? 0 : m_nframes);
-	}
-
-	// zero-crossing aliasing (median).
-	uint32_t zero_crossing ( uint32_t i ) const
-	{
-		uint32_t sum = 0;
-		for (uint16_t k = 0; k < m_nchannels; ++k)
-			sum += zero_crossing_k(i, k);
-		return (sum / m_nchannels);
-	}
+		{ return !m_pframes || (frame >= m_nframes); }
 
 private:
 
@@ -224,25 +193,20 @@ public:
 	// begin.
 	void start(void)
 	{
-		m_phase  = 1.0f;
-		m_phase1 = float(m_sample ? m_sample->length() : 0.0f);
-		m_index  = 1;
-		m_alpha  = 0.0f;
-		m_frame  = 0;
+		m_phase = 0.0f;
+		m_index = 0;
+		m_alpha = 0.0f;
+		m_frame = 0;
 	}
 
 	// iterate.
 	void next(float freq)
 	{
+		const float delta = freq * m_sample->ratio();
+
 		m_index  = int(m_phase);
 		m_alpha  = m_phase - float(m_index);
-		m_phase += freq * m_sample->ratio();
-
-		if (m_phase >= m_phase1) {
-			m_phase -= m_phase1;
-			if (m_phase < 1.0f)
-				m_phase = 1.0f;
-		}
+		m_phase += delta;
 
 		if (m_frame < m_index)
 			m_frame = m_index;
@@ -256,10 +220,10 @@ public:
 
 		const float *frames = m_sample->frames(k);
 
-		const float x0 = frames[m_index - 1];
-		const float x1 = frames[m_index];
-		const float x2 = frames[m_index + 1];
-		const float x3 = frames[m_index + 2];
+		const float x0 = frames[m_index];
+		const float x1 = frames[m_index + 1];
+		const float x2 = frames[m_index + 2];
+		const float x3 = frames[m_index + 3];
 
 		const float c1 = (x2 - x0) * 0.5f;
 		const float b1 = (x1 - x2);
@@ -280,7 +244,6 @@ private:
 	drumkv1_sample *m_sample;
 
 	float    m_phase;
-	float    m_phase1;
 	uint32_t m_index;
 	float    m_alpha;
 	uint32_t m_frame;
