@@ -1,7 +1,7 @@
 // drumkv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -133,8 +133,11 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Fla1WetKnob->setSpecialValueText(sOff);
 	m_ui.Pha1WetKnob->setSpecialValueText(sOff);
 	m_ui.Del1WetKnob->setSpecialValueText(sOff);
+	m_ui.Rev1WetKnob->setSpecialValueText(sOff);
 
-	m_ui.Del1BpmKnob->setSpecialValueText(tr("Auto"));
+	const QString& sAuto = tr("Auto");
+	m_ui.Gen1EnvTimeKnob->setSpecialValueText(sAuto);
+	m_ui.Del1BpmKnob->setSpecialValueText(sAuto);
 
 	// GEN group limits. [0=off, 1..128]
 	m_ui.Gen1GroupKnob->setSpecialValueText(sOff);
@@ -180,9 +183,10 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
 
 	// GEN1
-	setParamKnob(drumkv1::GEN1_GROUP,  m_ui.Gen1GroupKnob);
-	setParamKnob(drumkv1::GEN1_COARSE, m_ui.Gen1CoarseKnob);
-	setParamKnob(drumkv1::GEN1_FINE,   m_ui.Gen1FineKnob);
+	setParamKnob(drumkv1::GEN1_GROUP,   m_ui.Gen1GroupKnob);
+	setParamKnob(drumkv1::GEN1_COARSE,  m_ui.Gen1CoarseKnob);
+	setParamKnob(drumkv1::GEN1_FINE,    m_ui.Gen1FineKnob);
+	setParamKnob(drumkv1::GEN1_ENVTIME, m_ui.Gen1EnvTimeKnob);
 
 	// DCF1
 	setParamKnob(drumkv1::DCF1_CUTOFF,   m_ui.Dcf1CutoffKnob);
@@ -348,6 +352,9 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(drumkv1::OUT1_PANNING, m_ui.Out1PanningKnob);
 	setParamKnob(drumkv1::OUT1_VOLUME,  m_ui.Out1VolumeKnob);
 
+	// Reverb (stereo-)width limits.
+	m_ui.Rev1WidthKnob->setMinimum(-1.0f);
+	m_ui.Rev1WidthKnob->setMaximum(+1.0f);
 
 	// Effects
 	setParamKnob(drumkv1::CHO1_WET,   m_ui.Cho1WetKnob);
@@ -376,6 +383,13 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(valueChanged(float)),
 		SLOT(bpmSyncChanged()));
 
+	// Reverb
+	setParamKnob(drumkv1::REV1_WET,   m_ui.Rev1WetKnob);
+	setParamKnob(drumkv1::REV1_ROOM,  m_ui.Rev1RoomKnob);
+	setParamKnob(drumkv1::REV1_DAMP,  m_ui.Rev1DampKnob);
+	setParamKnob(drumkv1::REV1_FEEDB, m_ui.Rev1FeedbKnob);
+	setParamKnob(drumkv1::REV1_WIDTH, m_ui.Rev1WidthKnob);
+
 		// Dynamics
 	setParamKnob(drumkv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(drumkv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
@@ -452,6 +466,8 @@ drumkv1widget::drumkv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 // Param kbob (widget) map accesors.
 void drumkv1widget::setParamKnob ( drumkv1::ParamIndex index, drumkv1widget_knob *pKnob )
 {
+	pKnob->setDefaultValue(drumkv1_param::paramDefaultValue(index));
+
 	m_paramKnobs.insert(index, pKnob);
 	m_knobParams.insert(pKnob, index);
 
@@ -467,13 +483,14 @@ drumkv1widget_knob *drumkv1widget::paramKnob ( drumkv1::ParamIndex index ) const
 
 
 // Param port accessors.
-void drumkv1widget::setParamValue ( drumkv1::ParamIndex index, float fValue )
+void drumkv1widget::setParamValue (
+	drumkv1::ParamIndex index, float fValue, bool bDefault )
 {
 	++m_iUpdate;
 
 	drumkv1widget_knob *pKnob = paramKnob(index);
 	if (pKnob)
-		pKnob->setValue(fValue);
+		pKnob->setValue(fValue, bDefault);
 
 	updateParamEx(index, fValue);
 
@@ -636,7 +653,7 @@ void drumkv1widget::resetParamValues ( uint32_t nparams )
 	for (uint32_t i = 0; i < nparams; ++i) {
 		drumkv1::ParamIndex index = drumkv1::ParamIndex(i);
 		float fValue = drumkv1_param::paramDefaultValue(index);
-		setParamValue(index, fValue);
+		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
 	}
@@ -717,8 +734,8 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 
 	clearSampleFile();
 
-	resetParamValues(drumkv1::NUM_PARAMS);
 	resetParamKnobs(drumkv1::NUM_PARAMS);
+	resetParamValues(drumkv1::NUM_PARAMS);
 
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
@@ -750,14 +767,17 @@ void drumkv1widget::loadPreset ( const QString& sFilename )
 							drumkv1::ParamIndex index = drumkv1::ParamIndex(
 								eParam.attribute("index").toULong());
 							const QString& sName = eParam.attribute("name");
-							if (!sName.isEmpty() && s_hash.contains(sName))
+							if (!sName.isEmpty()) {
+								if (!s_hash.contains(sName))
+									continue;
 								index = s_hash.value(sName);
+							}
 							float fValue = eParam.text().toFloat();
 						//--legacy support < 0.3.0.4 -- begin
 							if (index == drumkv1::DEL1_BPM && fValue < 3.6f)
 								fValue *= 100.0f;
 						//--legacy support < 0.3.0.4 -- end.
-							setParamValue(index, fValue);
+							setParamValue(index, fValue, true);
 							updateParam(index, fValue);
 							m_params_ab[index] = fValue;
 						}
